@@ -21,10 +21,21 @@
 
 """Access to hooks."""
 
+__all__ = [
+    "CommitMsgShellHook",
+    "Hook",
+    "PostCommitShellHook",
+    "PostReceiveShellHook",
+    "PreCommitShellHook",
+    "PreReceiveShellHook",
+    "ShellHook",
+    "UpdateShellHook",
+]
+
 import os
 import subprocess
-from collections.abc import Sequence
-from typing import Any, Callable, Optional
+from collections.abc import Callable, Sequence
+from typing import Any
 
 from .errors import HookError
 
@@ -58,9 +69,9 @@ class ShellHook(Hook):
         name: str,
         path: str,
         numparam: int,
-        pre_exec_callback: Optional[Callable[..., Any]] = None,
-        post_exec_callback: Optional[Callable[..., Any]] = None,
-        cwd: Optional[str] = None,
+        pre_exec_callback: Callable[..., Any] | None = None,
+        post_exec_callback: Callable[..., Any] | None = None,
+        cwd: str | None = None,
     ) -> None:
         """Setup shell hook definition.
 
@@ -162,7 +173,7 @@ class CommitMsgShellHook(ShellHook):
 
             return (path,)
 
-        def clean_msg(success: int, *args: str) -> Optional[bytes]:
+        def clean_msg(success: int, *args: str) -> bytes | None:
             if success:
                 with open(args[0], "rb") as f:
                     new_msg = f.read()
@@ -191,7 +202,7 @@ class PostReceiveShellHook(ShellHook):
 
     def execute(
         self, client_refs: Sequence[tuple[bytes, bytes, bytes]]
-    ) -> Optional[bytes]:
+    ) -> bytes | None:
         """Execute the post-receive hook.
 
         Args:
@@ -226,9 +237,130 @@ class PostReceiveShellHook(ShellHook):
             out_data, err_data = p.communicate(in_data)
 
             if (p.returncode != 0) or err_data:
-                err_fmt = b"post-receive exit code: %d\n" + b"stdout:\n%s\nstderr:\n%s"
-                err_msg = err_fmt % (p.returncode, out_data, err_data)
-                raise HookError(err_msg.decode("utf-8", "backslashreplace"))
+                err_msg = (
+                    f"post-receive exit code: {p.returncode}\n"
+                    f"stdout:\n{out_data.decode('utf-8', 'backslashreplace')}\n"
+                    f"stderr:\n{err_data.decode('utf-8', 'backslashreplace')}"
+                )
+                raise HookError(err_msg)
             return out_data
         except OSError as err:
             raise HookError(repr(err)) from err
+
+
+class PreReceiveShellHook(ShellHook):
+    """pre-receive shell hook."""
+
+    def __init__(self, controldir: str) -> None:
+        """Initialize pre-receive hook.
+
+        Args:
+            controldir: Path to the git control directory (.git)
+        """
+        self.controldir = controldir
+        filepath = os.path.join(controldir, "hooks", "pre-receive")
+        ShellHook.__init__(self, "pre-receive", path=filepath, numparam=0)
+
+    def execute(
+        self, client_refs: Sequence[tuple[bytes, bytes, bytes]]
+    ) -> tuple[bytes, bytes]:
+        """Execute the pre-receive hook.
+
+        Args:
+            client_refs: List of tuples containing (old_sha, new_sha, ref_name)
+                        for each reference to be updated
+
+        Returns:
+            Tuple of (stdout, stderr) from hook execution
+
+        Raises:
+            HookError: If hook execution fails (exits with non-zero status)
+        """
+        # do nothing if the script doesn't exist
+        if not os.path.exists(self.filepath):
+            return (b"", b"")
+
+        try:
+            env = os.environ.copy()
+            env["GIT_DIR"] = self.controldir
+
+            p = subprocess.Popen(
+                self.filepath,
+                stdin=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+                env=env,
+            )
+
+            # client_refs is a list of (oldsha, newsha, ref)
+            in_data = b"\n".join([b" ".join(ref) for ref in client_refs])
+
+            out_data, err_data = p.communicate(in_data)
+
+            if p.returncode != 0:
+                raise HookError(
+                    f"pre-receive hook exited with status {p.returncode}: {err_data.decode('utf-8', 'backslashreplace')}"
+                )
+            return (out_data, err_data)
+        except OSError as err:
+            raise HookError(repr(err)) from err
+
+
+class UpdateShellHook(ShellHook):
+    """update shell hook."""
+
+    def __init__(self, controldir: str) -> None:
+        """Initialize update hook.
+
+        Args:
+            controldir: Path to the git control directory (.git)
+        """
+        self.controldir = controldir
+        filepath = os.path.join(controldir, "hooks", "update")
+        ShellHook.__init__(self, "update", path=filepath, numparam=3)
+
+    def execute(
+        self, ref_name: bytes, old_sha: bytes, new_sha: bytes
+    ) -> tuple[bytes, bytes]:
+        """Execute the update hook for a single ref.
+
+        Args:
+            ref_name: Name of the reference being updated
+            old_sha: Old SHA of the reference
+            new_sha: New SHA of the reference
+
+        Returns:
+            Tuple of (stdout, stderr) from hook execution
+
+        Raises:
+            HookError: If hook execution fails (exits with non-zero status)
+        """
+        # do nothing if the script doesn't exist
+        if not os.path.exists(self.filepath):
+            return (b"", b"")
+
+        try:
+            env = os.environ.copy()
+            env["GIT_DIR"] = self.controldir
+
+            p = subprocess.Popen(
+                [
+                    self.filepath,
+                    ref_name.decode("utf-8", "backslashreplace"),
+                    old_sha.decode("utf-8", "backslashreplace"),
+                    new_sha.decode("utf-8", "backslashreplace"),
+                ],
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+                env=env,
+            )
+
+            out_data, err_data = p.communicate()
+
+            if p.returncode != 0:
+                raise HookError(
+                    f"update hook exited with status {p.returncode}: {err_data.decode('utf-8', 'backslashreplace')}"
+                )
+            return (out_data, err_data)
+        except OSError as err:
+            raise HookError(repr(err)) from err
