@@ -52,6 +52,7 @@ __all__ = [
     "COMMAND_DEEPEN_NOT",
     "COMMAND_DEEPEN_SINCE",
     "COMMAND_DONE",
+    "COMMAND_FILTER",
     "COMMAND_HAVE",
     "COMMAND_SHALLOW",
     "COMMAND_UNSHALLOW",
@@ -84,6 +85,7 @@ __all__ = [
     "extract_capabilities",
     "extract_capability_names",
     "extract_want_line_capabilities",
+    "find_capability",
     "format_ack_line",
     "format_capability_line",
     "format_cmd_pkt",
@@ -101,6 +103,7 @@ __all__ = [
     "write_info_refs",
 ]
 
+import logging
 import types
 from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
 from io import BytesIO
@@ -112,6 +115,8 @@ import dulwich
 from .errors import GitProtocolError, HangupException
 from .objects import ObjectID
 
+logger = logging.getLogger(__name__)
+
 if TYPE_CHECKING:
     from .pack import ObjectContainer
     from .refs import Ref
@@ -174,6 +179,7 @@ CAPABILITY_ALLOW_REACHABLE_SHA1_IN_WANT = b"allow-reachable-sha1-in-want"
 CAPABILITY_FETCH = b"fetch"
 CAPABILITY_FILTER = b"filter"
 CAPABILITY_OBJECT_FORMAT = b"object-format"
+CAPABILITY_PACKFILE_URIS = b"packfile-uris"
 
 # Magic ref that is used to attach capabilities to when
 # there are no refs. Should always be ste to ZERO_SHA.
@@ -201,6 +207,8 @@ KNOWN_UPLOAD_CAPABILITIES = set(
         CAPABILITY_ALLOW_TIP_SHA1_IN_WANT,
         CAPABILITY_ALLOW_REACHABLE_SHA1_IN_WANT,
         CAPABILITY_FETCH,
+        CAPABILITY_FILTER,
+        CAPABILITY_PACKFILE_URIS,
     ]
 )
 KNOWN_RECEIVE_CAPABILITIES = set(
@@ -308,6 +316,7 @@ COMMAND_UNSHALLOW = b"unshallow"
 COMMAND_DONE = b"done"
 COMMAND_WANT = b"want"
 COMMAND_HAVE = b"have"
+COMMAND_FILTER = b"filter"
 
 
 def format_cmd_pkt(cmd: bytes, *args: bytes) -> bytes:
@@ -397,6 +406,24 @@ class Protocol:
         """Close the underlying transport if a close function was provided."""
         if self._close:
             self._close()
+            self._close = None  # Prevent double-close
+
+    def __del__(self) -> None:
+        """Ensure transport is closed when Protocol is garbage collected."""
+        if self._close is not None:
+            import warnings
+
+            warnings.warn(
+                f"unclosed Protocol {self!r}",
+                ResourceWarning,
+                stacklevel=2,
+                source=self,
+            )
+            try:
+                self.close()
+            except Exception:
+                # Ignore errors during cleanup
+                pass
 
     def __enter__(self) -> "Protocol":
         """Enter context manager."""
@@ -433,6 +460,7 @@ class Protocol:
             if size == 0 or size == 1:  # flush-pkt or delim-pkt
                 if self.report_activity:
                     self.report_activity(4, "read")
+                logger.debug("git< %s", sizestr.decode("ascii"))
                 return None
             if self.report_activity:
                 self.report_activity(size, "read")
@@ -446,6 +474,13 @@ class Protocol:
                 raise GitProtocolError(
                     f"Length of pkt read {len(pkt_contents) + 4:04x} does not match length prefix {size:04x}"
                 )
+            # Log the packet contents (truncate if too long for readability)
+            if len(pkt_contents) > 80:
+                logger.debug(
+                    "git< %s... (%d bytes)", pkt_contents[:80], len(pkt_contents)
+                )
+            else:
+                logger.debug("git< %s", pkt_contents)
             return pkt_contents
 
     def eof(self) -> bool:
@@ -498,6 +533,14 @@ class Protocol:
             prefix.
         """
         try:
+            # Log before converting to pkt format
+            if line is None:
+                logger.debug("git> 0000")
+            elif len(line) > 80:
+                logger.debug("git> %s... (%d bytes)", line[:80], len(line))
+            else:
+                logger.debug("git> %s", line)
+
             line = pkt_line(line)
             self.write(line)
             if self.report_activity:
@@ -721,6 +764,40 @@ def ack_type(capabilities: Iterable[bytes]) -> int:
     return SINGLE_ACK
 
 
+def find_capability(
+    capabilities: Iterable[bytes], *capability_names: bytes
+) -> bytes | None:
+    """Find a capability value in a list of capabilities.
+
+    This function looks for capabilities that may include arguments after an equals sign
+    and returns only the value part (after the '='). For capabilities without values,
+    returns the capability name itself.
+
+    Args:
+      capabilities: List of capability strings
+      capability_names: Capability name(s) to search for
+
+    Returns:
+      The value after '=' if found, or the capability name if no '=', or None if not found
+
+    Example:
+      >>> caps = [b'filter=blob:none', b'agent=git/2.0', b'thin-pack']
+      >>> find_capability(caps, b'filter')
+      b'blob:none'
+      >>> find_capability(caps, b'thin-pack')
+      b'thin-pack'
+      >>> find_capability(caps, b'missing')
+      None
+    """
+    for cap in capabilities:
+        for name in capability_names:
+            if cap == name:
+                return cap
+            elif cap.startswith(name + b"="):
+                return cap[len(name) + 1 :]
+    return None
+
+
 class BufferedPktLineWriter:
     """Writer that wraps its data in pkt-lines and has an independent buffer.
 
