@@ -134,6 +134,7 @@ def self_test_suite() -> unittest.TestSuite:
         "bitmap",
         "blackbox",
         "bundle",
+        "bundle_uri",
         "client",
         "cloud_gcs",
         "commit_graph",
@@ -189,6 +190,7 @@ def self_test_suite() -> unittest.TestSuite:
         "stash",
         "stripspace",
         "submodule",
+        "subtree",
         "trailers",
         "utils",
         "walk",
@@ -216,6 +218,7 @@ def self_test_suite() -> unittest.TestSuite:
         "notes",
         "rebase",
         "submodule",
+        "subtree",
         "tag",
         "worktree",
     ]
